package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.service.utils.AdmissionUtil;
import com.bocloud.sms.entity.Admission;
import com.bocloud.sms.interfaces.AdmissionService;
import com.bocloud.sms.model.AdmissionModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * ip访问限制控制器
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/12
 */
@RestController
@RequestMapping("/v1/admissions")
@Api(value = "IP限制管理", tags = "IP限制管理")
public class AdmissionController {

    private final AdmissionService admissionService;
    private final AdmissionUtil admissionUtil;

    public AdmissionController(AdmissionService admissionService, AdmissionUtil admissionUtil) {
        this.admissionService = admissionService;
        this.admissionUtil = admissionUtil;
    }

    /**
     * 获取IP访问限制列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询IP访问限制列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Admission>> list(Pager pager) {
        return admissionService.list(pager);
    }

    /**
     * 添加IP访问限制
     *
     * @param admissionModel
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建IP访问限制", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody AdmissionModel admissionModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Assert.notNull(admissionModel, "请求参数不能为空");
        Result result = admissionService.create(admissionModel, requestContext);
        if (result.isSuccess()) {
            admissionUtil.refresh();
        }
        return result;
    }

    /**
     * 删除IP访问限制
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除IP访问限制", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        Result result = admissionService.remove(id);
        if (result.isSuccess()) {
            admissionUtil.refresh();
        }
        return result;
    }

    /**
     * 查看IP访问限制详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看IP访问限制", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Admission> detail(@PathVariable(value = Common.ID) Long id) {
        return admissionService.detail(id);
    }

    /**
     * 批量删除IP访问限制
     *
     * @param ids
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量删除IP访问限制", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping
    public Result batchRemove(@RequestBody List<Long> ids) {
        return admissionService.batchRemove(ids);
    }

}
