package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.CloudVendor;
import com.bocloud.sms.interfaces.CloudVendorService;
import com.bocloud.sms.model.CloudVendorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 云供应商Controller
 */
@Api(value = "云供应商", tags = "云供应商")
@RestController
@RequestMapping("/v1/vendors")
public class CloudVendorController {

    @Autowired
    private CloudVendorService cloudVendorService;

    /**
     * 获取云供应商列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取云供应商列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<CloudVendor>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.list(pager, context);
    }

    /**
     * 获取云供应商列表
     *
     * @param condition
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "通过condition方式获取云平台列表", httpMethod = "GET")
    @GetMapping("/condition")
    public BocloudResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.condition(condition, context);
    }

    /**
     * 添加云供应商
     *
     * @param vendor
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "创建云供应商", httpMethod = "POST")
    @PostMapping
    public BocloudResult<CloudVendor> create(@RequestBody CloudVendor vendor, @Value(Common.REQ_CONTEXT) RequestContext context) {
        vendor.setCreatorId(context.getTarget());
        vendor.setMenderId(context.getTarget());
        return cloudVendorService.create(vendor, context);
    }

    /**
     * 修改云供应商
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改云供应商信息", httpMethod = "PUT")
    @PutMapping("/{id}")
    public BocloudResult<CloudVendor> modify(@PathVariable(value = Common.ID) Long id, @RequestBody CloudVendorBean bean,
                                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return cloudVendorService.modify(bean, context.getTarget());
    }

    /**
     * 移除云供应商
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "移除云供应商信息", httpMethod = "DELETE")
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.remove(id, context);
    }

    /**
     * 查看云供应商详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查看云供应商详细信息", httpMethod = "GET")
    @GetMapping("/{id}")
    public BocloudResult<CloudVendor> detail(@PathVariable(value = Common.ID) Long id) {
        return cloudVendorService.detail(id);
    }

    /**
     * 冻结云供应商
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "云平台操作", httpMethod = "PATCH")
    @PatchMapping("/{id}")
    public BocloudResult action(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = Common.ACTION, required = false) String action,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.action(id, action, context);
    }

    /**
     * 更新认证信息
     *
     * @param id
     * @param cloudVendor
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改云平台密码", httpMethod = "PATCH")
    @PatchMapping("/{id}/password")
    public BocloudResult updatePassword(@PathVariable(value = Common.ID) Long id, @RequestBody CloudVendor cloudVendor,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.updatePassword(id, cloudVendor, context);
    }


    /**
     * 批量开启/关闭告警
     *
     * @param ids
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "批量开启/关闭告警", httpMethod = "PATCH")
    @PatchMapping("/alarm")
    public BocloudResult batchOperationAlarm(@RequestBody List<Long> ids,
                                             @RequestParam(value = Common.ACTION) String action,
                                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.batchOperationAlarm(ids, action, context.getTarget());
    }


}
