package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.License;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.license.LicenseDaemon;
import com.bocloud.sms.model.LicenseMask;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

/**
 * 服务节点信息
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1")
@Api(value = "服务节点信息管理", tags = "服务节点信息管理")
public class ConfigController {

    private static final String LICENSE_SUCCESS = "证书激活成功";
    private static final String LICENSE_FAILED = "证书激活失败";
    private static final Logger logger = LoggerFactory.getLogger(ConfigController.class);
    @Value("${service.host:127.0.0.1}")
    private String host;
    @Value("${server.port:18088}")
    private String port;
    private final LicenseDaemon licenseDaemon;

    public ConfigController(CurrentService currentService) {
        this.licenseDaemon = LicenseDaemon.instance(currentService);
    }


    /**
     * 查询云管license信息
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询云管license信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/config/license")
    public BocloudResult<License> license() {
        License license = licenseDaemon.content();
        return new BocloudResult<>(true, license, "查询成功");
    }

    /**
     * 系统状态信息
     *
     * @param value
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "系统状态信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/config/status")
    public BocloudResult status(@RequestParam(value = Common.VALUE) String value) {
        HttpClient client = new HttpClient();
        String message = client.get(value).getMessage();
        return JSONObject.parseObject(message, BocloudResult.class);
    }

    /**
     * 安装证书
     *
     * @param file
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "安装证书", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/config/license/install")
    public Result licenseInstall(@RequestParam("file") MultipartFile file) {
        if (file.isEmpty()) {
            return new Result(false, "证书为空!");
        }
        Result result;
        try {
            result = licenseDaemon.verify(file.getBytes());
            if (result.isFailed()) {
                logger.error("证书校验失败");
                return new Result(false, LICENSE_FAILED);
            }
            result = licenseDaemon.install(file.getBytes());
            if (result.isSuccess()) {
                return new Result(true, LICENSE_SUCCESS);
            }
            return new Result(false, LICENSE_FAILED);
        } catch (IOException e) {
            logger.error("证书激活异常：", e);
            return new Result(false, LICENSE_FAILED);
        }
    }

    /**
     * 安装license验证码
     *
     * @param licenseMask
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "安装license验证码", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/config/captcha/install")
    public Result captchaInstall(@RequestBody LicenseMask licenseMask) {
        try {
            BocloudResult<byte[]> bocloudResult = licenseDaemon.decode(licenseMask.getLicenseMask());
            if (bocloudResult.isFailed()) {
                logger.error("证书激活码格式有误，激活失败!");
                return new Result(false, LICENSE_FAILED);
            }
            Result result = licenseDaemon.verify(bocloudResult.getData());
            if (result.isFailed()) {
                logger.error("证书格式不正确!");
                return new Result(false, LICENSE_FAILED);
            }
            result = licenseDaemon.install(bocloudResult.getData());
            if (result.isFailed()) {
                logger.error("证书安装失败");
                return new Result(false, LICENSE_FAILED);
            }
            return new Result(true, LICENSE_SUCCESS);
        } catch (Exception e) {
            logger.error("证书激活异常！", e);
            return new Result(false, LICENSE_FAILED);
        }
    }

    /**
     * 系统sid信息
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "系统状态信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/status/sid")
    public BocloudResult getSid() {
        HttpClient client = new HttpClient();
        String url = "http://" + host + ":" + port + "/status/sid";
        String message = client.get(url).getMessage();
        return JSONObject.parseObject(message, BocloudResult.class);
    }

    /**
     * 系统sid信息
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "系统状态信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/status/sids")
    public BocloudResult getSids() {
        HttpClient client = new HttpClient();
        String url = "http://" + host + ":" + port + "/status/sids";
        String message = client.get(url).getMessage();
        return JSONObject.parseObject(message, BocloudResult.class);
    }
}
