package com.bocloud.sms.booter.controller;

import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.Tokens;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.repository.SystemConfigRepository;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Objects;
import java.util.Optional;

/**
 * @author tyl
 * @time 2020/4/24 周五 20:02
 */
@Controller
@RequestMapping
@Api(value = "跳转控制", tags = "跳转控制")
@Slf4j
public class DispatchController {

    private final SystemConfigRepository configRepository;
    private final StringRedisTemplate redisTemplate;
    private final Logger logger = LoggerFactory.getLogger(DispatchController.class);

    public DispatchController(SystemConfigRepository systemConfigService, StringRedisTemplate redisTemplate) {
        this.configRepository = systemConfigService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 单点登录跳转接口
     *
     * @param request
     * @param response
     * @throws Exception
     */

    @RequestMapping("/v1/redirect")
    @ApiOperation(value = "单点登录跳转",tags = {"CSC","CMC"})
    public void redirect(HttpServletRequest request, HttpServletResponse response) throws Exception {
        //获取登录地址
        String target = request.getParameter("target");
        String redirectParameter = request.getParameter("redirect");
        SystemConfig config;
        if (target == null || "Manager".equals(target)) {
            config = configRepository.queryByCode("ManagerAddress");
        } else {
            config = configRepository.queryByCode("TenantAddress");
        }
        String loginAddress = config.getValue();
        logger.info("#######debuge########the request url is {}", request.getRequestURL() + "?" + request.getQueryString());
        logger.info("#######debuge########the redirect url is {}", redirectParameter);
        String redirect = Optional.ofNullable(redirectParameter).orElse(loginAddress + "/#/");
        String token = request.getParameter("token");
        logger.info("#######debuge########the redirect token is {}", token);
        Claims claims = Tokens.parse(token);
        if (!Objects.isNull(claims)) {
            String catalog = claims.get("catalog", String.class);
            Long uuid = claims.get(Common.UUID, Long.class);
            String tokenKey = Common.TOKEN + "_" + catalog + "_" + uuid;
            boolean checkResult = check(token, tokenKey);
            if (checkResult) {
                HttpSession session = request.getSession();
                session.setAttribute("redirect", redirect);
                session.setAttribute(Common.TOKEN, token);
                response.sendRedirect(loginAddress + "/#/sso");
            } else {
                String replace = redirect.replace("#", "%23");
                response.sendRedirect(loginAddress + "/#/login?redirect=" + replace);
            }
        } else {
            String replace = redirect.replace("#", "%23");
            response.sendRedirect(loginAddress + "/#/login?redirect=" + replace);
        }
    }

    private boolean check(String token, String tokenKey) {
        String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
        String oldTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_o");
        if (token.equals(newTokenCache)) {
            return true;
        } else {
            return token.equals(oldTokenCache);
        }
    }
}
