package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.AccessLog;
import com.bocloud.sms.interfaces.LogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 审计日志控制器，用来进行平台审计日志的查询操作
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/logs")
@Api(value = "审计日志管理", tags = "审计日志管理")
public class LogController {

    private final LogService logService;

    public LogController(LogService logService) {
        this.logService = logService;
    }

    /**
     * 审计日志查询
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取审计日志列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<AccessLog>> list(Pager pager) {
        return this.logService.list(pager);

    }

    /**
     * 批量保存审计日志
     *
     * @param logs 审计日志数组
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量保存审计日志", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result batchSave(@RequestBody List<AccessLog> logs) {
        return this.logService.batch(logs);
    }

    /**
     * 统计服务调用次数
     *
     * @return
     */
    @GetMapping(value = "/stats")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "统计服务调用次数", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<JSONObject> statistics() {
        return this.logService.statistics();
    }

    /**
     * 审计日志归档
     *
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param deleted 是否删除
     * @param context 请求对象
     * @return
     */
    @GetMapping(value = "/archive")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "审计日志归档", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<StringBuilder> archive(@RequestParam(value = "startTime", required = false) String startTime,
                                                @RequestParam(value = "endTime", required = false) String endTime,
                                                @RequestParam(value = "deleted", required = false) Boolean deleted,
                                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                                                HttpServletResponse response) {
        return logService.archive(startTime, endTime, deleted, context, request, response);
    }

}
