package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.NameRule;
import com.bocloud.sms.interfaces.NameRuleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 命名规则控制器
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/7/2
 */
@RestController
@RequestMapping("/v1/namings")
@Api(value = "命名规则", tags = "命名规则")
public class NameRuleController {

    private final NameRuleService nameRuleService;

    public NameRuleController(NameRuleService nameRuleService) {
        this.nameRuleService = nameRuleService;
    }

    /**
     * 获取命名规则列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取命名规则列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<NameRule>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nameRuleService.list(pager, context);
    }

    /**
     * 添加命名规则
     *
     * @param nameRule
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加命名规则", httpMethod = "POST")
    @PostMapping
    public BocloudResult<NameRule> create(@RequestBody NameRule nameRule,
                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nameRuleService.create(nameRule, context);
    }

    /**
     * 修改命名规则
     *
     * @param id
     * @param nameRule
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改命名规则", httpMethod = "PUT")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody NameRule nameRule, @Value(Common.REQ_CONTEXT) RequestContext context) {
        nameRule.setId(id);
        return nameRuleService.modify(nameRule, context);
    }

    /**
     * 根据命名规则生成云主机名称
     *
     * @param id
     * @param tenantId
     * @param userId
     * @param projectId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据命名规则生成云主机名称", httpMethod = "PATCH")
    @PatchMapping("/{id}")
    public BocloudResult<String> patch(@PathVariable(value = Common.ID) Long id,
                                       @RequestParam(value = "tenantId") Long tenantId,
                                       @RequestParam(value = "userId") Long userId,
                                       @RequestParam(value = "projectId") Long projectId, RequestContext context) {
        return nameRuleService.generate(id, tenantId, userId, projectId, context);
    }

    /**
     * 删除命名规则
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除命名规则", httpMethod = "DELETE")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return nameRuleService.remove(id);
    }

    /**
     * 查看命名规则详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看命名规则详情", httpMethod = "GET")
    @GetMapping("/{id}")
    public BocloudResult<NameRule> detail(@PathVariable(value = Common.ID) Long id) {
        return nameRuleService.detail(id);
    }

}
