package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Plugins;
import com.bocloud.sms.interfaces.PluginsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 插件表
 *
 * @author fanzx
 * @version 5.6.0-SNAPSHOT
 * @since 2022/2/16
 */
@RestController
@RequestMapping("/v1/plugins")
@Api(value = "plugins插件", tags = "plugins插件")
public class PluginsController {

    @Autowired
    private PluginsService pluginsService;

    /*-------------------------------------------- 不可见插件列表接口 --------------------------------------------*/
    /**
     * 新增Plugin插件
     * @param plugins   插件对象
     * @param context
     * @return  成功/失败
     */
    @PostMapping
    @ApiOperation(value = "新增Plugin插件", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody Plugins plugins,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.create(plugins, context);
    }

    /**
     * 修改Plugin插件
     * @param plugins   插件对象
     * @param context
     * @return  成功/失败
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改Plugin插件", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody Plugins plugins,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.modify(id,plugins, context);
    }

    /**
     * 删除Plugin插件
     * @param id    插件id
     * @param context
     * @return  成功/失败
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除Plugin插件", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.remove(id, context);
    }

    /**
     * 批量删除Plugin插件
     *
     * @param idList    插件id列表
     * @param context 上下文对象
     * @return 成功/失败 个数
     */
    @DeleteMapping("/remove")
    @ApiOperation(value = "批量删除Plugin插件", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult batchRemove(@RequestBody List<Long> idList,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.batchRemove(idList, context.getTarget());
    }

    /**
     * 获取Plugin插件列表
     * @param pager
     * @param context
     * @return  插件列表
     */
    @GetMapping
    @ApiOperation(value = "获取Plugin插件列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.list(pager, context);
    }

    /**
     * 获取Plugin插件单条详情
     * @param id    插件id
     * @return  对应插件详细信息
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "获取Plugin插件详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return pluginsService.detail(id);
    }

    /**
     * 获取Plugins支持的云平台列表
     * @param pager
     * @param context
     * @return  插件支持的云平台列表信息 for 纳管时使用
     */
    @GetMapping("/vendors")
    @ApiOperation(value = "获取Plugins支持的云平台列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult getVendorList(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.getVendorList(pager,context);
    }

    /**
     * 开启/关闭 插件
     *
     * @param plugins 插件对象
     * @param context   上下文对象
     * @return 布尔值 成功或者失败
     */
    @PatchMapping("/status")
    @ApiOperation(value = "插件状态控制", httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult enabled(@RequestBody Plugins plugins,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return pluginsService.enabled(plugins.getId(), plugins.getStatus(), context);
    }

    /*-------------------------------------------- 可见插件列表接口 --------------------------------------------*/

    /**
     * 获取license中的插件列表
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/license")
    @ApiOperation(value = "获取Plugin插件列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult licenseList(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context){
        return pluginsService.listLicense(pager,context);
    }
}
