package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.interfaces.SystemConfigService;
import com.bocloud.sms.model.ConfigModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @Version 1.0
 * @since 2017年8月18日
 */
@RestController
@RequestMapping("/v1")
@Api(value = "系统配置管理", tags = "系统配置管理")
public class SystemConfigController {

    private final SystemConfigService systemConfigService;

    public SystemConfigController(SystemConfigService systemConfigService) {
        this.systemConfigService = systemConfigService;
    }

    /**
     * 更新系统配置
     *
     * @param requestContext
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "更新系统配置", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/system-configs")
    public Result modify(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        List<ConfigModel> configs = JSONArray.parseArray(request.getParameter("configs"), ConfigModel.class);
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        return systemConfigService.batchUpdate(configs, fileMap, requestContext);
    }


    /**
     * 同步Ldap
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "同步Ldap", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/configs/syncLdap")
    public Result syncLdap(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return systemConfigService.syncLdap(requestContext);
    }

    /**
     * 查询系统配置
     *
     * @param category
     * @param codes
     * @return
     */
    @ApiOperation(value = "批量查询系统配置", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/configs")
    public BocloudResult<Map<String, String>> getByCodes(@RequestParam(value = "category" ,required = false) String category,@RequestParam(value = "codes",required = false) ArrayList<String> codes) {
        return systemConfigService.listConfigs(category,codes);
    }


    /**
     * 查询系统配置类型树
     *
     * @param category
     * @return
     */
    @ApiOperation(value = "查询系统配置类型树", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/configs/tree")
    public BocloudResult<JSONArray> listCategoryTree(@RequestParam(value = "category") String category) {
        return systemConfigService.listCategoryTree(category);
    }


    /**
     * 系统配置连接测试
     *
     * @param category
     * @return
     */
    @ApiOperation(value = "系统配置连接测试", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/configs/test")
    public Result configTest(@RequestParam(value = "category") String category) {
        return systemConfigService.configTest(category);
    }
}
