package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.SystemTask;
import com.bocloud.sms.interfaces.SystemTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 平台任务控制器
 *
 * @author tyl
 * @since 2021/4/1 18:22
 */
@RestController
@RequestMapping("/v1/system/tasks")
@Api(value = "平台任务管理", tags = "平台任务管理")
public class SystemTaskController {

    private final SystemTaskService systemTaskService;

    public SystemTaskController(SystemTaskService systemTaskService) {
        this.systemTaskService = systemTaskService;
    }

    /**
     * 查询平台任务列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "获取平台任务列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<SystemTask>> list(Pager pager) {
        return systemTaskService.list(pager);
    }

    /**
     * 添加平台任务
     *
     * @param systemTask
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "创建平台任务", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody SystemTask systemTask,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return systemTaskService.create(systemTask, requestContext);
    }

    /**
     * 修改平台任务
     *
     * @param systemTask
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "平台任务修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody SystemTask systemTask,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return systemTaskService.modify(id, systemTask, requestContext);
    }

    /**
     * 删除平台任务
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "平台任务删除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return systemTaskService.remove(id, requestContext);
    }

    /**
     * 查看平台任务详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "平台任务详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<SystemTask> detail(@PathVariable(value = Common.ID) Long id) {
        return systemTaskService.detail(id);
    }
}
