package com.bocloud.sms.booter.controller;

import com.bocloud.cmp.boot.core.utils.Tokens;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.interfaces.UserService;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Map;

/**
 * @author tyl
 * @since 2020/7/24 周五 11:38
 */
@RestController
@Api(value = "token管理", tags = "token管理")
public class TokenController {

    private final StringRedisTemplate redisTemplate;
    private final TenantService tenantService;
    private final UserService userService;

    public TokenController(StringRedisTemplate redisTemplate, TenantService tenantService, UserService userService) {
        this.redisTemplate = redisTemplate;
        this.tenantService = tenantService;
        this.userService = userService;
    }

    /**
     * 单点登录验证token
     *
     * @param request
     * @return
     */
    @GetMapping("/v1/token")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "单点登录验证token", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<String> checkToken(HttpServletRequest request) {
        String token = request.getHeader(Common.TOKEN);
        if (!StringUtils.hasText(token)) {
            token = request.getParameter(Common.TOKEN);
        }
        String tokenKey = "";
        if (StringUtils.hasText(token)) {
            Claims claims = com.bocloud.cmp.boot.core.utils.Tokens.parse(token);
            if (claims == null) {
                return new BocloudResult<>(false, "token校验失败");
            }
            String catalog = claims.get("catalog", String.class);
            Long uuid = claims.get(Common.UUID, Long.class);
            tokenKey = Common.TOKEN + "_" + catalog + "_" + uuid;
        }
        //校验token
        boolean checkResult = check(token, tokenKey);
        if (checkResult) {
            String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
            return new BocloudResult<>(true, newTokenCache, "校验token成功");
        } else {
            return new BocloudResult<>(false, "token校验失败");
        }
    }

    private boolean check(String token, String tokenKey) {
        String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
        if (token.equals(newTokenCache)) {
            return true;
        } else {
            String oldTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_o");
            return token.equals(oldTokenCache);
        }
    }

    @PostMapping("/v1/sso/token/info")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "token解析获取信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<Map<String, Object>> getTokenInfo(HttpServletRequest request) {
        String token = request.getHeader(Common.TOKEN);
        HttpSession session = request.getSession();
        if (!StringUtils.hasText(token)) {
            return new BocloudResult<>(false, "token不存在");
        }
        Claims parse = Tokens.parse(token);
        if (parse == null) {
            return new BocloudResult<>(false, "token信息异常重新登录");
        }
        String catalog = parse.get("catalog", String.class);
        String account = parse.get(Common.ACCOUNT, String.class);
        BocloudResult<Map<String, Object>> result;
        if (catalog.equals(RequestContext.Catalog.Tenant.name())) {
            result = tenantService.tokenLogin(account);
        } else {
            result = userService.tokenLogin(account);
        }
        Map<String, Object> object = result.getData();
        object.put(Common.TOKEN, token);
        object.put("redirect", session.getAttribute("redirect"));
        result.setData(object);
        return result;
    }
}
