package com.bocloud.sms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.sms.booter.websocket.message.MessagePublisher;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.stereotype.Component;

/**
 * 站内信通知监听
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年8月10日
 */
@Slf4j
@Component
public class BocloudMessageListener implements MessageListener {

    private final MessagePublisher publisher;

    public BocloudMessageListener(MessagePublisher publisher) {
        this.publisher = publisher;
    }

    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (log.isDebugEnabled()) {
                log.debug("get bocloud message:{}", content);
            }
            Assert.notNull(content, "消息内容为空");
            BocloudResult<JSONObject> result = JSONTools.isJSON(content);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            OperateResult operate = JSONObject.parseObject(content, OperateResult.class);
            Assert.notNull(content, "消息格式不正确");
            RequestContext context = operate.getContext();
            if (null != context.getTarget() && 0 != context.getTarget()) {
                // 发送消息给指定ID的用户前端
                publisher.pushMessage(context.getCatalog(), context.getTarget(), content);
            } else {// 发送给所有的用户前端
                publisher.pushMessage(content);
            }
        } catch (Exception e) {
            log.error("Get error message:{}", e.getMessage());
        }
    }
}
