package com.bocloud.sms.entity;

import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 审计日志信息类
 *
 * @author dmw
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Table("access_log")
public class AccessLog extends Generic {
    
    @PK(value = PKStrategy.UUID)
    private String id;
    @Column("target")
    private String target;
    @Column("action")
    private String action;
    @Column("result")
    private String result;
    @Column("user_id")
    private Long userId;
    @Column("response_ip")
    private String responseIp;
    @Column("request_ip")
    private String requestIp;
    @Column(value = "gmt_create", nullable = false)
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    @Column("cost")
    private Long cost;
    @Column("detail")
    private String detail;
    @Column("module")
    private String module;
    @Column("tenant_id")
    private Long tenantId;
    @Column("request_module")
    private String requestModule;
    @Column("catalog")
    private Catalog catalog;
    @IgnoreUpdate
    @IgnoreInsert
    private String account;
    @IgnoreAll
    private String userName;

    /**
     * @param id
     * @param target
     * @param action
     * @param result
     * @param userId
     * @param responseIp
     * @param requestIp
     * @param gmtCreate
     * @param detail
     * @param module
     * @param tenantId
     */
    public AccessLog(String id, String target, String action, String result, Long userId, String responseIp,
                     String requestIp, Date gmtCreate, String detail, String module, Long tenantId) {
        super();
        this.id = id;
        this.target = target;
        this.action = action;
        this.result = result;
        this.userId = userId;
        this.responseIp = responseIp;
        this.requestIp = requestIp;
        this.gmtCreate = gmtCreate;
        this.detail = detail;
        this.module = module;
        this.tenantId = tenantId;
    }

    /**
     * @param id
     * @param target
     * @param action
     * @param result
     * @param userId
     * @param responseIp
     * @param requestIp
     * @param gmtCreate
     * @param detail
     * @param module
     * @param tenantId
     * @param requestModule
     */
    public AccessLog(String id, String target, String action, String result, Long userId, String responseIp,
                     String requestIp, Date gmtCreate, String detail, String module, Long tenantId, String requestModule) {
        super();
        this.id = id;
        this.target = target;
        this.action = action;
        this.result = result;
        this.userId = userId;
        this.responseIp = responseIp;
        this.requestIp = requestIp;
        this.gmtCreate = gmtCreate;
        this.detail = detail;
        this.module = module;
        this.tenantId = tenantId;
        this.requestModule = requestModule;
    }
}