package com.bocloud.sms.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreUpdate;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 登录记录
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年1月29日
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("login_record")
public class LoginRecord extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id;// ID
    @Column("account")
    private String account;// 账号
    @Column("request_ip")
    private String requestIp;// 请求IP
    @Column("response_ip")
    private String responseIp;// 响应IP
    @Column("status")
    private Boolean status;// 登录状态
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;// 登录时间
    @Column("detail")
    private String detail;// 登录详情
    @Column("category")
    private Category category;

    public enum Category {
        /**
         * 租户
         */
        Tenant,
        /**
         * 用户
         */
        User,
        /**
         * 管理员
         */
        Manager
    }
}
