package com.bocloud.sms.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 管理员用户
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年1月29日
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("manager")
public class Manager extends GenericEntity {
    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 账号
     */
    @Column("account")
    private String account;
    /**
     * 密码
     */
    @Column("password")
    private String password;
    /**
     * 邮箱
     */
    @Column("email")
    private String email;
    /**
     * 电话
     */
    @Column("mobile")
    private String mobile;
    /**
     * 登录状态
     */
    @Column("login_status")
    private Boolean loginStatus;
    /**
     * 公司名称
     */
    @Column("company")
    private String company;
    @Column("session_id")
    private String sessionId;
    /**
     * 组织机构id
     */
    @Column("depart_id")
    private Long departId;
    @Column("depart_ids")
    private String departIds;
    /**
     * 工号
     */
    @Column("job_number")
    private String jobNumber;
    /**
     * 性别
     */
    @Column("sex")
    private Boolean sex;
    /**
     * 头像
     */
    @Column("portrait")
    private String portrait;
    /**
     * 来源(SYSTEM/LDAP)
     */
    @Column("origin")
    private String origin;
    /**
     * 最后一次密码修改时间
     */
    @Column("last_pwd_modify_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastPwdModifyDate;
    /**
     * 最后一次登录时间
     */
    @Column("last_login_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastLoginDate;
    /**
     * 过期时间
     */
    @Column("expired_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredDate;
    /**
     * 组织机构名称
     */
    @IgnoreAll
    private String departName;
    @IgnoreAll
    private Boolean checked;
    @IgnoreAll
    private List<String> roleNames;
    /**
     * 企业微信唯一标识userId
     */
    @Column("wechat")
    private String weChatWorkNumber;
    /**
     * 钉钉唯一标识userId
     */
    @Column("dingtalk")
    private String dingTalkNumber;
}
