package com.bocloud.sms.interfaces;

import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.sms.entity.AccountSecurity;
import com.bocloud.sms.entity.LoginRecord;
import com.bocloud.sms.entity.Manager;
import com.bocloud.sms.entity.Role;
import com.bocloud.sms.model.ActionModel;
import com.bocloud.sms.model.ManagerModel;
import com.bocloud.sms.model.PermissionBean;

import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @time 2020/2/4 周二 9:38
 */
public interface ManagerService {
    /**
     * 管理员登录
     *
     * @param account
     * @param password
     * @param sessionId
     * @param requestIp
     * @return
     * @throws InternalServerException
     */
    BocloudResult<Map<String, Object>> login(String account, String password, String sessionId, String requestIp) throws InternalServerException;

    /**
     * 管理员退出
     *
     * @param id
     * @return
     * @throws InternalServerException
     */
    Result logout(Long id) throws InternalServerException;

    /**
     * 创建管理员
     *
     * @param manager
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    Result create(ManagerModel manager, RequestContext requestContext) throws InternalServerException;

    /**
     * 更新管理员
     *
     * @param id
     * @param managerModel
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    Result modify(Long id, ManagerModel managerModel, RequestContext requestContext) throws InternalServerException;

    /**
     * 删除管理员
     *
     * @param id
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    Result remove(Long id, RequestContext requestContext) throws InternalServerException;

    /**
     * 修改管理员状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    Result updateStatus(Long id, ActionModel.Action action, RequestContext requestContext) throws InternalServerException;

    /**
     * 修改管理员密码
     *
     * @param id
     * @param oldPassword
     * @param password
     * @param requestContext
     * @return
     */
    Result changePwd(Long id, String oldPassword, String password, RequestContext requestContext);

    /**
     * 重置管理员密码
     *
     * @param id
     * @param password
     * @param requestContext
     * @return
     */
    Result resetPwd(Long id, String password, RequestContext requestContext);

    /**
     * 管理员授权角色
     *
     * @param id
     * @param roleIds
     * @param requestContext
     * @return
     */
    Result accredit(Long id, List<Long> roleIds, RequestContext requestContext);

    /**
     * 查看管理员详情
     *
     * @param id
     * @return
     */
    BocloudResult<Manager> detail(Long id);

    /**
     * 查询管理员列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    BocloudResult<GridBean<Manager>> list(Pager pager, RequestContext requestContext);

    /**
     * 获取安全密钥
     *
     * @param apiKey
     * @return
     */
    BocloudResult<AccountSecurity> getSecurity(String apiKey);

    /**
     * 校验密码
     *
     * @param id
     * @param password
     * @return
     */
    Result check(Long id, String password);

    /**
     * 检查管理员是否异地登录
     *
     * @param id
     * @param token
     * @return
     */
    Result checkManager(Long id, String token);

    /**
     * 查询管理员的角色
     *
     * @param id
     * @param requestContext
     * @return
     */
    BocloudResult<List<Role>> listRoles(Long id, RequestContext requestContext);

    /**
     * 根据管理员账号冻结管理员基本信息
     *
     * @param account
     * @return
     * @throws InternalServerException
     */
    Result lockByAccount(String account) throws InternalServerException;

    /**
     * 查询管理员权限树形列表
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<PermissionBean>> listPermissions(RequestContext requestContext);

    /**
     * 查询所有管理员
     *
     * @return
     */
    BocloudResult<List<Manager>> list();

    /**
     * 保存管理员数据
     *
     * @param log
     * @return
     * @throws InternalServerException
     */
    Result save(LoginRecord log) throws InternalServerException;

    /**
     * 管理员登录记录列表展示
     *
     * @param pager
     * @return
     */
    BocloudResult<GridBean<LoginRecord>> list(Pager pager);

    /**
     * 返回账号相关登录所需信息
     *
     * @param account
     * @return
     */
    BocloudResult<Map<String, Object>> tokenLogin(String account);

    /**
     * 根据账号查询管理员信息
     *
     * @param account
     * @return
     */
    Result exist(String account);

}
