package com.bocloud.sms.interfaces;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.model.PermissionBean;
import com.bocloud.sms.model.PermissionIcon;
import com.bocloud.sms.model.PermissionModel;
import com.bocloud.sms.model.PermissionParentsBean;

import java.util.List;

/**
 * @author tyl
 * @time 2020/2/4 周二 9:43
 */
public interface PermissionService {
    /**
     * 创建权限
     *
     * @param params
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result create(PermissionModel params, RequestContext requestContext) throws InternalServerException;

    /**
     * 修改权限
     *
     *
     * @param id
     * @param params
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result modify(Long id, PermissionModel params, RequestContext requestContext) throws InternalServerException;

    /**
     * 删除权限
     *
     * @param id
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result remove(Long id, RequestContext requestContext) throws InternalServerException;

    /**
     * 权限详情
     *
     * @param id
     * @return
     */
    BocloudResult<Permission> detail(Long id);

    /**
     * 查询权限
     *
     * @param parentId
     * @param category
     * @param tenant
     * @return
     */
    BocloudResult<List<PermissionBean>> list(Long parentId, String category, Boolean tenant);

    /**
     * 查询父节点
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<PermissionParentsBean>> listParents(RequestContext requestContext);

    /**
     * 查询图标
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<PermissionIcon>> listIcon(RequestContext requestContext);

    /**
     * 根据服务id查询服务菜单权限
     *
     * @param serviceId
     * @return
     */
    BocloudResult<Permission> getServicePermission(Long serviceId);

    /**
     * 根据服务id集合查询服务菜单权限
     *
     * @param services
     * @return
     */
    BocloudResult<List<Permission>> getServicePermissions(List<Long> services);

    /**
     * 获取指定API模块权限列表
     *
     * @return
     */
    BocloudResult<JSONArray> getApiList();

    BocloudResult<List<Permission>> listPermissionsByCondition(String condition);

    /**
     * 插件绑定权限
     * @param permissionIdList  权限id列表
     * @param pluginsCode       插件编码
     * @param tenant            是否是租户端的菜单页面
     * @param requestContext
     * @return
     */
    Result bindingPlugins(List<Long> permissionIdList, String pluginsCode, Boolean tenant, RequestContext requestContext);
}
