package com.bocloud.sms.interfaces;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.sms.entity.Plugins;

import java.util.List;

/**
 * 插件表
 *
 * @author fanzx
 * @version 5.6.0-SNAPSHOT
 * @since 2022/2/16
 */
public interface PluginsService {

    /**
     * 查询列表
     *
     * @param pager
     * @param context
     * @return plugins列表
     */
    BocloudResult list(Pager pager, RequestContext context);

    /**
     * 添加
     *
     * @param plugins
     * @param context
     * @return 成功/失败
     */
    BocloudResult create(Plugins plugins, RequestContext context);

    /**
     * 更新
     *
     * @param id
     * @param plugins 实体类
     * @param context
     * @return 更新 成功/失败
     */
    BocloudResult modify(Long id, Plugins plugins, RequestContext context);

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 删除
     */
    BocloudResult remove(Long id, RequestContext context);

    /**
     * 批量删除
     * @param idList
     * @param target
     * @return  成功/失败 个数
     */
    BocloudResult batchRemove(List<Long> idList, Long target);

    /**
     * 获取详细信息
     *
     * @param id
     * @return plugins详细信息
     */
    BocloudResult detail(Long id);

    /**
     * 查询插件下的云平台列表
     * @param pager
     * @param context
     * @return 云平台列表 to ims
     */
    BocloudResult getVendorList(Pager pager, RequestContext context);

    /**
     * 控制插件状态
     * @param id        插件id
     * @param status    插件状态
     * @param context
     * @return  成功/失败
     */
    BocloudResult enabled(Long id, Boolean status, RequestContext context);

    /**
     * 获取license中的插件列表
     * @param pager
     * @param context
     * @return
     */
    BocloudResult listLicense(Pager pager, RequestContext context);
}
