package com.bocloud.sms.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.LogArchiveRecord;
import com.bocloud.sms.interfaces.LogArchiveRecordService;
import com.bocloud.sms.repository.LogArchiveRecordRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author admin
 */
@Service("logArchiveRecordService")
public class LogArchiveRecordServiceImpl implements LogArchiveRecordService {

    private static final Logger logger = LoggerFactory.getLogger(LogArchiveRecordServiceImpl.class);
    private final LogArchiveRecordRepository logArchiveRecordRepository;

    public LogArchiveRecordServiceImpl(LogArchiveRecordRepository logArchiveRecordRepository) {
        this.logArchiveRecordRepository = logArchiveRecordRepository;
    }

    @Override
    public BocloudResult<GridBean<LogArchiveRecord>> list(Pager pager) {
        GridBean<LogArchiveRecord> gridBean;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtArchive", Common.ONE);
            int total = this.logArchiveRecordRepository.count(params);
            List<LogArchiveRecord> list = this.logArchiveRecordRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query archive log record list error:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

}
