package com.bocloud.sms.service.internel;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Cmp内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月15日
 */
@Component
public class CmpInternalService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static final Service SERVICE = Service.create(BoCloudService.IMS);
    private final Map<String, Object> headers = new HashMap<>(4);

    public CmpInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    public BocloudResult detailPool(Long poolId) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/pool/groups/" + poolId,
                BoCloudMethod.GET, headers, null);
        return remoteService.invoke();
    }

    public BocloudResult tenantCheck(List<Long> poolIds, Long tenantId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "tenantCheck");
        object.put("poolIds", poolIds);
        object.put("tenantId", tenantId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/pool/groups",
                BoCloudMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    public BocloudResult createOpProject(List<Long> poolIds, Long tenantId, String apiKey) {
        this.headers.put(Common.API_KEY, apiKey);
        JSONObject object = new JSONObject();
        object.put("poolIds", poolIds);
        object.put("tenantId", tenantId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vendors/op/projects", BoCloudMethod.POST, headers, object);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    @SuppressWarnings("unused")
    public BocloudResult deleteOpProject(Long tenantId) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vendors/op/projects",
                BoCloudMethod.DELETE, headers, MapTools.simpleMap(Common.TENANTID, tenantId));
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());

    }
}
