package com.bocloud.sms.service.internel;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.bocloud.sms.entity.ResourceQuota;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * cos服务用户内部访问
 *
 * @author weiwei
 */
@Component("cosQuotaService")
public class CosQuotaService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static final Service SERVICE = Service.create(BoCloudService.COS);
    private final Map<String, Object> headers = new HashMap<>(4);

    public CosQuotaService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 删除配额
     *
     * @param target  目标id
     * @param catalog 账号类型
     * @return
     */
    public BocloudResult delete(Long target, String catalog) {
        String url = "/v1/quotas";
        JSONObject object = new JSONObject();
        object.put("target", target);
        object.put("catalog", catalog.toUpperCase());
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.DELETE, headers, params);
        return remoteService.invoke();
    }

    /**
     * 查询配额信息
     *
     * @param target  目标id
     * @param catalog 账号类型
     * @return
     */
    public BocloudResult quotas(Long target, String catalog, String apiKey) {
        String url = "/v1/tenants/" + target + "/quotas";
        JSONObject object = new JSONObject();
        object.put("target", target);
        object.put("catalog", catalog);
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        headers.put(Common.API_KEY, apiKey);
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.GET, headers, params);
        return remoteService.invoke();
    }

    /**
     * 查询租户项目配额信息
     *
     * @param projectIds 项目id集合
     * @return
     */
    public Map<String, Long> projectQuotas(List<Long> projectIds) {
        String url = "/v1/quotas";
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "getProjectsQuotas");
        object.put("projectIds", projectIds);
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.GET, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isSuccess() && result.getData() != null) {
            HashMap<String, Long> resultMap = new HashMap<>(1);
            JSONArray jsonArray = JSONArray.parseArray(JSONObject.toJSONString(result.getData()));
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                String code = jsonObject.getString("code");
                Long quotaTotal = jsonObject.getLong("quotaTotal");
                resultMap.put(code, quotaTotal);
            }
            return resultMap;
        }
        return null;
    }

    /**
     * 查询配额信息
     *
     * @param target  目标id
     * @param catalog 账号类型
     * @return
     */
    public BocloudResult basicQuotas(Long target, String catalog, String apiKey) {
        String url = "/v1/tenants/" + target + "/quotas";
        JSONObject object = new JSONObject();
        object.put("target", target);
        object.put("catalog", catalog);
        object.put(Common.CONDITION, "basicQuota");
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        headers.put(Common.API_KEY, apiKey);
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.GET, headers, params);
        return remoteService.invoke();
    }

    /**
     * 跟新释放配额
     *
     * @param quotas
     * @param apiKey
     * @return
     */
    public BocloudResult releaseQuota(List<ResourceQuota> quotas, String apiKey) {
        String url = "/v1/quotas";
        headers.put(Common.API_KEY, apiKey);
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, JSONObject.toJSONString(quotas));
        params.put(Common.ACTION, ResourceQuota.Action.Release.name());
        params.put("projectId", 0);
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.PATCH, headers, params);
        return remoteService.invoke();
    }

    /**
     * 添加配额
     *
     * @param quotas 资源配额集合
     * @param target 租户id
     * @return
     */
    public BocloudResult assign(List<ResourceQuota> quotas, Long target, String apiKey) {
        String url = "/v1/tenants/" + target + "/quotas";
        headers.put(Common.API_KEY, apiKey);
        RemoteService remoteService = serviceFactory.build(SERVICE, url, BoCloudMethod.POST, headers,
                MapTools.simpleMap(Common.PARAMS, JSONArray.toJSONString(quotas)));
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
