package com.bocloud.sms.service.ldap;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.Manager;
import com.bocloud.sms.entity.User;
/*import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;*/
import org.springframework.util.StringUtils;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.ArrayList;
import java.util.List;

public class LdapManagerRepository {

    private final LdapConnector connector;

    public LdapManagerRepository(LdapConnector connector) {
        super();
        this.connector = connector;
    }

    private DirContext getContext() {
        return this.connector.getConnector();
    }

    private String getBaseDn() {
        return this.connector.getBaseDn();
    }

    public boolean save(String dn, Manager manager) {
        Attributes attributes = new BasicAttributes();
        BasicAttribute attribute = new BasicAttribute("objectclass");
        attribute.add("top");
        attribute.add("organizationalPerson");
        attributes.put(attribute);
        attributes.put("uid", manager.getAccount());
        attributes.put("ou", manager.getDepartName());
        attributes.put("cn", manager.getName());
        attributes.put("password", manager.getPassword());
        if (StringUtils.hasText(manager.getRemark())) {
            attributes.put("description", manager.getRemark());
        }
        String name = dn + "," + this.getBaseDn();
        try {
            this.getContext().createSubcontext(name, attributes);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public boolean delete(String dn) {
        try {
            this.getContext().destroySubcontext(dn);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

/*    public User query(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.OBJECT_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalPerson"));
        try {
            NamingEnumeration<SearchResult> results = this.getContext().search(dn + "," + this.getBaseDn(),
                    filter.encode(), sc);
            JSONObject manager = new JSONObject();
            while (results.hasMore()) {
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("ou".equals(attribute.getID())) {
                            manager.put("account", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            manager.put("remark", attribute.get());
                        }
                    }
                }
            }
            return manager.isEmpty() ? null : JSONObject.parseObject(manager.toJSONString(), User.class);
        } catch (NamingException e) {
            return null;
        }
    }*/

    public boolean update(String oldDn, Department oldDept, String newDn, Department newDept) {
        try {
            List<ModificationItem> items = new ArrayList<>();
            if (!oldDn.equals(newDn)) {
                this.getContext().rename(oldDn + "," + this.getBaseDn(), newDn + "," + this.getBaseDn());
                Attribute attribute = new BasicAttribute("ou", newDept.getName());
                items.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attribute));
            }
            if (!StringUtils.hasText(oldDept.getRemark()) && StringUtils.hasText(newDept.getRemark())) {
                String description = !StringUtils.hasText(newDept.getRemark()) ? "" : newDept.getRemark();
                Attribute attribute = new BasicAttribute("description", description);
                items.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attribute));
            }
            if (items.isEmpty()) {
                return true;
            }
            this.getContext().modifyAttributes(newDn + "," + this.getBaseDn(),
                    items.toArray(new ModificationItem[0]));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

/*    public List<User> list(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.ONELEVEL_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalPerson"));
        try {
            String base = !StringUtils.hasText(dn) ? this.getBaseDn() : dn + "," + this.getBaseDn();
            NamingEnumeration<SearchResult> results = this.getContext().search(base, filter.encode(), sc);
            List<User> list = new ArrayList<>();
            while (results.hasMore()) {
                JSONObject manager = new JSONObject();
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("cn".equals(attribute.getID())) {
                            manager.put("name", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            manager.put("remark", attribute.get());
                        } else if ("uid".equals(attribute.getID())) {
                            manager.put("account", attribute.get());
                        } else if ("password".equals(attribute.getID())) {
                            manager.put("password", attribute.get());
                        } else if ("ou".equals(attribute.getID())) {
                            manager.put("departName", attribute.get());
                        }
                    }
                }
                if (!manager.isEmpty()) {
                    list.add(JSONObject.parseObject(manager.toJSONString(), User.class));
                }
            }
            return list;
        } catch (NamingException e) {
            return null;
        }
    }*/

}
