package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 审计日志数据迁移任务（这个数据量实在是太大了，单独拿出来容易监控进度）
 *
 * @author luyanchao
 * @date 2021-06-17
 */
//@Component 数据量太大，暂时先注释掉
public class AccessLogTask implements UpgradeTask{

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    public AccessLogTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "审计日志数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        List<String> columns = UpgradeUtils.getColumns(this.dataSource, "access_log");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "access_log", columns.toArray(new String[0]),
                percent -> this.percent = Math.floorDiv(percent * 80, 100));
        this.percent = 80;
        // 处理userId
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "access_log", "user_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 20, idMap.size()) + 80;
            index++;
        }
        this.percent = 100;
    }
}
