package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 分发策略
 *
 * @author zhangyf
 * @date 2021-07-19
 */
@Component
public class AlarmSendPolicyTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 因为有用户ID
     */
    @Autowired
    public AlarmSendPolicyTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "分发策略迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "alarm_send_policy", "monitor_alarm_send_policy", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;

        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMapOfUser.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm_send_policy", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm_send_policy", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm_send_policy", "is_weixin", null, "0");
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm_send_policy", "is_dingding", null, "0");
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm_send_policy", "native_alarm", null, "0");
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[]{"id", "name", "wait_time", "is_urgent", "is_primary", "is_secondary",
            "is_remind", "is_message", "is_email", "is_action", "block_policy", "gmt_create", "gmt_modify", "creator_id",
            "mender_id", "tenant_id"
    };
}
