package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * 公有云账单数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-06
 */
@Component
public class CloudBillTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     */
    @Autowired
    public CloudBillTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "公有云账单数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "cloud_bill", COLUMNS,
            percent -> this.percent = percent);
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS = new String[] {"id", "order_id", "sn", "status", "money", "gmt_create",
        "month", "user_id", "project_id", "tenant_id", "params", "vendor_id", "resource_id", "product_code",
        "product_detail", "product_name", "fee_begin_time", "fee_end_time", "vendor_type", "order_sn", "charge_mode"};

    // 5.5增加字段: 华为，阿里，azure使用，都可留空
    // consumed_service------------------------------------------------------azure
    // quantity--------------------------------------------------------------azure
    // unit_price------------------------------------------------------------azure
    // subscription_id-------------------------------------------------------azure
    //
    // deducted_byCashoupons-------------------------------------------------公有云账单
    // deducted_byCoupons----------------------------------------------------公有云账单
    // deducted_byPrepaidCard------------------------------------------------公有云账单
    // invoice_discount------------------------------------------------------公有云账单
    // outstanding_amount----------------------------------------------------公有云账单
    // pretax_amount---------------------------------------------------------公有云账单
    // pretax_gross_amount---------------------------------------------------公有云账单
    // round_down_discount---------------------------------------------------公有云账单
    // subscription_type-----------------------------------------------------公有云账单
    // region_id-------------------------------------------------------------公有云账单
    // total_cost------------------------------------------------------------公有云账单
    // tenant_money----------------------------------------------------------租户金额，由totalCost计算得出

}
