package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 服务目录数据迁移任务
 *
 * 直接舍弃5.0的数据，保留5.5的数据
 * @author luyanchao
 * @date 2021-06-23
 */
@Component
public class CloudServiceTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    public CloudServiceTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "服务目录数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            // 处理creator_id、mender_id、owner_id
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_service", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_service", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_service", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 100, idMap.size());
            index++;
        }
        this.percent = 100;
    }
}
