package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 云平台统计数据迁移任务
 *
 * 新增的字段无需处理，定时任务会自动更新上去的
 *
 * @author luyanchao
 * @date 2021-06-21
 */
@Component
public class CloudVendorStatisticsTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public CloudVendorStatisticsTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "云平台数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 先把数据从5.0迁移到5.5
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "cloud_vendor_statistics", COLUMN, percent -> this.percent = percent);
        this.percent = 100;
    }

    private static final String[] COLUMN = new String[] {
            "access_path_count","alias_count","cluster_count","cpu_count","cpu_total","cpu_unused","cpu_used",
            "data_store_count","deployment_count","disk_total","disk_unused","disk_used","edge_cluster_count",
            "fabric_count","file_client_count","file_system_count","file_user_count","firewall_count",
            "flash_copy_count","group_count","host_count","id","image_count","instance","logic_port_count",
            "lpar_count","mdisk_count","mds_count","mem_total","mem_unused","mem_used","mon_count","monitor_count",
            "network_count","node_count","ns_count","object_bucket_count","object_router_count","object_user_count",
            "offline","online","osd_count","pod_count","pool_count","profile_count","router_count","service_count",
            "snapshot_count","switch_count","template_count","tenant_count","transport_zone_count","vas_count",
            "vendor_id","vios_count","vpc_count","vs_count","zone_count"
    };
}
