package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.CloudVendorServiceImpl;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 云平台数据迁移任务
 *
 * @author luyanchao
 * @date 2021-06-21
 */
@Component
public class CloudVendorTask implements UpgradeTask {

    private static final String[] COLUMN = new String[]{
            "authentication", "availablitiy_zone", "creator_id", "dc_id", "deploy_password", "deploy_url", "deploy_username",
            "docker_address", "env_id", "gmt_create", "gmt_modify", "guid", "harbor_id", "id", "is_alarm", "is_deleted",
            "is_public", "location", "logo", "mender_id", "name", "owner_id", "props", "region", "remark", "remote_site",
            "status", "supplier", "sync_cron", "type", "uuid", "vendor_category", "version"
    };
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public CloudVendorTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "云平台数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 先把数据从5.0迁移到5.5
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "cloud_vendor", COLUMN, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_vendor", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_vendor", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_vendor", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        // 将CloudVendor数据写入zk
        CloudVendorServiceImpl cloudVendorService = context.getBean(CloudVendorServiceImpl.class);
        try {
            cloudVendorService.updateProvider(cloudVendorService.listAll());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
