package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * 规格数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-06
 */
@Component
public class FlavorTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     */
    @Autowired
    public FlavorTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "规格数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "flavor", COLUMNS,
            percent -> this.percent = percent);
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS =
        new String[] {"id", "name", "flavor_uuid", "memory", "cpu", "disk", "vendor_id", "family", "status",
            "instance_charge_type", "zone", "region_id", "extra_specs_id", "performance_type"};

    // 5.5增加字段:attachable_quantity、availability_zone_ids、available、cpu_architecture、os_extra_specs
}
