package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 流程节点迁移
 *
 * @author zhangyf
 * @date 2021-07-19
 */
@Component
public class FlowNodeTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 因为有用户ID
     */
    @Autowired
    public FlowNodeTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "流程节点迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "wf_node",
                percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;

        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfUser(this.dataSource, this.context.getBean(DataSource.class));
        Map<Long, Long> idMapOfManager = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));

        NamedParameterJdbcTemplate targetJdbcTemplate = new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMapOfUser.entrySet()) {
            Map<String, Object> sqlParams = new HashMap<>();
            sqlParams.put("oldId", id.getKey());
            sqlParams.put("newId", id.getValue());
            targetJdbcTemplate.update("update wf_node set operator_id = :newId where operator_catalog = 'User' and operator_id = :oldId", sqlParams);
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size() + idMapOfManager.size()) + 50;
            index++;
        }
        for (Map.Entry<Long, Long> id : idMapOfManager.entrySet()) {
            Map<String, Object> sqlParams = new HashMap<>();
            sqlParams.put("oldId", id.getKey());
            sqlParams.put("newId", id.getValue());
            targetJdbcTemplate.update("update wf_node set operator_id = :newId, operator_catalog = 'User' where operator_catalog = 'Manager' and operator_id = :oldId", sqlParams);
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size() + idMapOfManager.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

}
