package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 镜像数据迁移任务
 *
 * @author luyanchao
 * @date 2021-07-01
 */
@Component
public class ImageTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public ImageTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "角色数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        // 先把数据从5.0迁移到5.5
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "image", COLUMN, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "image", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "image", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "image", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        //  5.5增加了如下字段
        // block_device_mappings 无从考证，留null
        // offer 和李振平确认，留null
        // publisher  和李振平确认，留null
        // resource_group_uuid 无从考证，留null
        // root_device_name 无从考证，留null
        // root_device_type 无从考证，留null
        // sku 和李振平确认，留null
        // support 无从考证，留null
        // virtual_env_type 无从考证，留null
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    private static final String[] COLUMN = new String[] {
            "architecture","azure_resource_group_name","catalog","creator_id","default_user","file_path","gmt_create",
            "gmt_modify","icon_id","id","image_label","image_type","image_uuid","is_deleted","iso_name",
            "kernel_version","mender_id","min_disk","name","os_category","os_name","os_type","os_version","owner_id",
            "project_id","props","qga_installed","region","region_id","remark","size","status","store_id","template_id",
            "tenant_id","tenant_uuid","vendor_id","vendor_type"
    };
}
