package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;

/**
 * 历史登录记录数据迁移任务
 *
 * 1. 该表不能走UnchangeTask这个任务，因为5.5中必然会有登录记录，迁移5.0的时候就有可能有id冲突
 * 2. 从5.0迁移到5.5的时候不迁移id，id通过自增自动生成
 *
 * @author luyanchao
 * @date 2021-06-18
 */
@Component
public class LoginRecordTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private DataSource dataSource;

    @Autowired
    public LoginRecordTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public String getName() {
        return "历史登录记录数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "login_record", COLUMNS, percent -> this.percent = percent);
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[] {
            "account","request_ip","response_ip","status","detail","gmt_create","category"
    };
}
