package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * network_card数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-13
 */
@Component
public class NetworkCardTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     * @param userTask
     *            依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public NetworkCardTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "network_card数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "network_card", COLUMNS,
            percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "network_card", "creator_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "network_card", "mender_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "network_card", "owner_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS = new String[] {"id", "name", "mac", "used", "creator_id", "host_id",
        "gmt_create", "gmt_modify", "mender_id", "owner_id", "status", "props", "remark", "is_deleted", "vendor_id",
        "category", "label", "switcher_id", "port_type", "pci_id", "region_id", "mtu", "number", "zone", "vpc_uuid",
        "subnet_uuid", "ip_address", "networkcard_uuid", "is_primary"};

    // 5.5增加字段:
    // floatingip_uuid
    // instance_id

}
