package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * nsx分段数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-06
 */
@Component
public class NsxSegmentTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * 
     * @param context
     *            spring上下文
     * @param userTask
     *            依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public NsxSegmentTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "nsx分段数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "nsx_segment", COLUMNS,
            percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "nsx_segment", "creator_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "nsx_segment", "mender_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "nsx_segment", "owner_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS = new String[] {"id", "name", "segment_id", "links", "resource_schema",
        "self", "revision", "create_time", "create_user", "last_modified_time", "last_modified_user", "protection",
        "system_owned", "resource_type", "tags", "parent_path", "path", "relative_path", "children",
        "marked_for_delete", "advanced_config", "connectivity_path", "domain_name", "l2_extension", "ls_id",
        "overlay_id", "subnets", "transport_zone_path", "type", "vlan_ids", "switch_id", "vendor_id", "gmt_create",
        "gmt_modify", "creator_id", "owner_id", "mender_id", "status", "is_deleted", "props", "remark"};

    // 5.5增加字段:dhcp_config_id、vpc_id

}
