package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: PortalElementPoolTask
 * @Description: 首页配置数据迁移任务
 * @Author lxy
 * @Date 2021/8/9 11:59
 */
@Component
public class PortalPanelTask implements UpgradeTask {
    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    public PortalPanelTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "首页配置数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate v55jdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "portal_panel", COLUMNS,
                percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "portal_panel", "creator_id", id.getKey(),
                    id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "portal_panel", "mender_id", id.getKey(),
                    id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        //获取所有数据--处理url中所有带有cmp的数据
        String sqlUpdate = "update portal_panel set config = replace ( config ,\"/cmp/v1\",\"/ims/v1\" )";
        v55jdbcTemplate.update(sqlUpdate, new HashMap<>());
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[]{
            "id", "config", "location", "server_name", "mender_id", "gmt_modify", "is_deleted", "gmt_create", "creator_id",
            "tenant_id", "project_id"
    };
}
