package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;

/**
 * role_permission数据迁移任务
 * 1. 维持超级管理员角色的权限不变（拥有所有权限）
 * 2. 删掉其他角色中已授予的但是5.5中删除了的权限
 * @author luyanchao
 * @date 2021-06-09
 */
@Component
public class RolePermissionTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private DataSource dataSource;

    /**
     * 构造方法
     *
     * 暂时不依赖其他任务，因为role的id不会变，PermissionTask任务重排permissionId后同时处理了role_permission表
     * @param context spring上下文
     */
    @Autowired
    public RolePermissionTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public String getName() {
        return "角色授权数据迁移任务";
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = this.context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 删掉5.5中非超级管理员之外的角色授权
        String sqlDelete = "delete from role_permission where role_id != 1";
        targetJdbcTemplate.update(sqlDelete, new HashMap<>());
        this.percent = 10;

        // 迁移角色授权数据(超级管理员除外)
        String sqlSelect = "select role_id, permission_id from role_permission where role_id != 1 ";
        String sqlInsert = "insert into role_permission (role_id, permission_id) values (:role_id, :permission_id) ";
        List<Map<String, Object>> fromDbDatas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        int index = 1;
        for (Map<String, Object> data : fromDbDatas) {
            targetJdbcTemplate.update(sqlInsert, data);
            this.percent = Math.floorDiv(index * 80, fromDbDatas.size()) + 10;
            index++;
        }
        this.percent = 90;

        // 删掉角色中已授予的，但是5.5中删除了的权限
        String sqlClear = "delete from role_permission where permission_id not in (select id from permission) ";
        targetJdbcTemplate.update(sqlClear, new HashMap<>());
        this.percent = 100;
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }
}
