package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;

/**
 * 云主机标签数据迁移
 *
 * @author luyanchao
 * @date 2021-07-15
 */
@Component
public class ServerLabelTask implements UpgradeTask {

    private final ApplicationContext context;
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context spring上下文
     */
    @Autowired
    public ServerLabelTask(ApplicationContext context) {
        this.context = context;

    }

    @Override
    public String getName() {
        return "云主机标签数据迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "server_label", COLUMNS, percent -> this.percent = percent);
        this.percent = 100;
        //-------------------------------------5.5增加了如下字段---------------------------------
        // content --已经和袁明航确认留null即可

    }

    private static final String[] COLUMNS = new String[]{
            "id", "target", "label_id", "type", "vendor_id"
    };
}
