package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 宿主机数据迁移任务
 *
 * @author luyanchao
 * @date 2021-07-01
 */
@Component
public class ServerTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public ServerTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "宿主机数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 先把数据从5.0迁移到5.5
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "server", COLUMN, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "server", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "server", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "server", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        //  ------------------- 5.5增加了如下字段 -------------------
        //  discover_user_id -- 已经和卢艳超确认留null即可
        //  enabled -- 已经和元明航确认留null即可
    }

    private static final String[] COLUMN = new String[] {
            "asset_category","catalog","category","clock_diff","cluster_id","cpu_model","creator_id","dc_id",
            "discover_time","discovered","expired_time","exporter_installed","gmt_create","gmt_modify","guid",
            "hostname","id","identifier","image_id","instance","ipv6_enabled","is_accurated","is_connected",
            "is_deleted","is_maintained","manage_ip","mender_id","model_name","name","os","os_category","owner_id",
            "power_state","props","rack_id","rackhd_node_id","remark","resource_group","resource_id","resource_label",
            "role","room_id","root_disk_id","service_ip","services","status","tenant_id","uuid","vdc_id","vendor_id",
            "vmotion_ip"
    };
}
