package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @since 2021/7/8 14:27
 */
@Component
public class ServiceOrderTask implements UpgradeTask {
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    public ServiceOrderTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        dependOn.add(userTask);
    }

    @Override public String getName() {
        return "订单数据迁移任务";
    }

    @Override public int getPercent() {
        return this.percent;
    }

    @Override public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * 相比较5.0版本 5.5去除了租户id字段 同时还需要处理迁移过程中的user_id变化
     */
    @Transactional(rollbackFor = Throwable.class) @Override public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "service_order", SERVICE_ORDER,
            percent -> this.percent = Math.floorDiv(percent * 60, 100));
        this.percent = 60;
        // 处理userId
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "service_order", "user_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 15, idMap.size()) + 60;
            index++;
        }
        // 查询5.0的项目 更新业务订单
        NamedParameterJdbcTemplate v50jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        NamedParameterJdbcTemplate v55jdbcTemplate = new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        String projectSql = "select id, business_id from project where is_deleted = 0 and business_id is not null";
        List<Map<String, Object>> projectResults = v50jdbcTemplate.queryForList(projectSql, new HashMap<>());
        int projectIndex = 1;
        for (Map<String, Object> projectResult : projectResults) {
            long projectId = Long.parseLong(projectResult.get("id").toString());
            long businessId = Long.parseLong(projectResult.get("business_id").toString());
            Map<String, Object> sqlParams = MapTools.simpleMap("projectId", projectId);
            sqlParams.put("businessId", businessId);
            String sqlUpdate = "update service_order set business_id = :businessId where project_id = :projectId ";
            v55jdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(projectIndex * 15, projectResult.size()) + 75;
            projectIndex++;
        }
        String sqlUpdate = "update service_order set order_type = 'apply' ";
        v55jdbcTemplate.update(sqlUpdate, new HashMap<>());
        this.percent = 100;
    }

    private static final String[] SERVICE_ORDER =
        new String[] {"id","sn", "charge_mode", "sla_agreed", "resource_status", "order_status", "user_id",
            "project_id", "tenant_id", "coupon_id", "is_deleted", "gmt_create", "gmt_modify", "remarks", "message",
            "apply_param", "service_code", "auto_renew", "vendor_id"};
}
