package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 存储主机表数据迁移
 *
 * @author wangshiyao
 * @date 2021-06-30
 */
@Component
public class StorageHostTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造函数
     * @param context spring上下文
     * @param userTask  creator_id、mender_id、owner_id字段依赖
     */
    @Autowired
    public StorageHostTask(ApplicationContext context, UserTask userTask){
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "存储主机表数据迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Asserts.notNull(dataSource, "dataSource can't be null" );
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class) ,
                "storage_host", COLUMNS, percent->this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        //处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for(Map.Entry<Long, Long> id : idMapOfUser.entrySet()){
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class),"storage_host" , "creator_id", id.getKey() , id.getValue() );
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "storage_host", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "storage_host", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size()) + 50;
            index++;
        }
        this.percent = 100;
        //-------------------------------------5.5增加了如下字段---------------------------------
        //is_virtual 为Dell存储增加，5.0时候没有Dell存储，所以留null即可
    }

    private static final String[] COLUMNS = new String[]{"chap" , "chap_name" , "chap_secret" , "creator_id" , "description" , "gmt_create" , "gmt_modify" ,
            "host_uuid" , "id" , "is_deleted" , "iscsi_port_id" , "key_path" , "mender_id" , "name" , "owner_id" , "props" ,
            "remark" , "status" , "storage_id" , "type" , "vendor_id"
    };
}
