package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务模板数据迁移任务
 *
 * 5.5和5.0相比未增减初始化数据，但是graph和is_maintain字段值发生了变化
 *
 * @author luyanchao
 * @date 2021-07-14
 */
@Component
public class TaskTemplateTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public TaskTemplateTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "任务模板数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 查询5.5数据
        String sqlSelect55 = "select id, remark from task_template where 1=1 ";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 清除数据·
        String sqlTruncate = "delete from task_template";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "task_template", percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 按5.5更新graph和is_maintain
        int index = 1;
        String sqlUpdate = "update task_template set graph = :graph, is_maintain = :is_maintain where id = :id ";
        for (Map<String, Object> v55Data : v55Datas) {
            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("graph", v55Data.get("graph"));
            paramUpdate.put("is_maintain", v55Data.get("is_maintain"));
            paramUpdate.put("id", v55Data.get("id"));
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            this.percent = Math.floorDiv(index * 25, v55Datas.size()) + 50;
            index++;

        }
        this.percent = 75;
        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task_template", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task_template", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task_template", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 25, idMap.size()) + 75;
            index++;
        }
        this.percent = 100;
        // 5.5增加了gmt_execute字段，已和魏伟确认留null
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }
}
