package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @since 2021/7/8 14:27
 */
@Component
public class UserProjectTask implements UpgradeTask {
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    public UserProjectTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "项目数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * 相比较5.0版本 5.5去除了租户id字段 同时还需要处理迁移过程中的user_id变化
     */
    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 迁移项目数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "user_project", USER_PROJECT_COLUMNS, percent -> this.percent = Math.floorDiv(percent * 80, 100));
        this.percent = 80;
        // 处理userId
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfUser(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "user_project", "user_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 20, idMap.size()) + 80;
            index++;
        }
        this.percent = 100;
    }

    private static final String[] USER_PROJECT_COLUMNS = new String[]{
            "user_id", "project_id", "project_admin"
    };
}
