package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 卷数据迁移
 *
 * @author wangshiyao
 * @date 2021-06-30
 */
@Component
public class VolumeTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public VolumeTask(ApplicationContext context, UserTask userTask){
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "卷数据迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null" );
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "volume", COLUMNS, percent->this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;

        //处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for(Map.Entry<Long, Long> id : idMapOfUser.entrySet()){
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class),"volume" , "creator_id", id.getKey() , id.getValue() );
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "volume", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "volume", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size()) + 50;
            index++;
        }

        this.percent = 100;
        //-------------------------------------5.5增加了如下字段---------------------------------
        // backup_count --已经和袁明航确认留null即可
        // vaults_uuid  --已经和袁明航确认留null即可
        // business_id  --已经和袁明航确认留null即可
        // expired_time ---已经和袁明航确认留null即可
        // for_use      --- 为EMC存储增加，5.0时候没有EMC存储，所以留EMC即可
        // resource_group_id ---已经和李振平确认留null即可

    }

    private static final String[] COLUMNS = new String[]{"access_path_id", "allocated_size", "attached_time",
            "azure_resource_group_name", "block_count", "block_size", "boot_volume", "catalog", "category",
            "compressed", "create_time", "creator_id", "data_store_id", "delete_auto_snapshot", "delete_with_instance",
            "detached_time", "device", "disk_charge_type", "disk_expire_time", "disk_object_id", "enable_auto_snapshot",
            "file_system", "force_mount", "gmt_create", "gmt_modify", "id", "identifier", "image_uuid", "instance_id",
            "iops", "is_deleted", "key_path", "lvm_mntp", "mender_id", "migration_stripes", "multiattach", "name",
            "operational_status", "owner_id", "performance_priority", "pool_id", "project_id", "props", "qos",
            "qos_enabled", "region", "region_id", "remark", "request_id", "sample", "server_id", "size", "sku_id",
            "snapshot_id", "source_snapshot_id", "status", "storage_id", "sync_rate", "synced_set_id", "template_id",
            "tenant_id", "tenant_uuid", "thinly_provisioned", "type", "udid", "uuid", "vendor_id", "volume_type",
            "volume_type_id", "volume_uuid", "zone"

    };
}
