package com.bocloud.sms.service.utils;

import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.repository.AccountSecurityRepository;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.ManagerRepository;
import com.bocloud.sms.repository.SystemConfigRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.concurrent.TimeUnit;

/**
 * 定时同步数据类
 *
 * @author weiwei
 */
@Component
public class SyncScheduler implements InitializingBean {

    private final ManagerRepository managerRepository;
    private final DepartmentRepository departmentRepository;
    private final SystemConfigRepository systemConfigRepository;
    private final AccountSecurityRepository accountSecurityRepository;
    private final LockFactory lockFactory;
    private final CurrentService currentService;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final Logger logger = LoggerFactory.getLogger(SyncScheduler.class);

    public SyncScheduler(ManagerRepository managerRepository, DepartmentRepository departmentRepository, SystemConfigRepository systemConfigRepository, AccountSecurityRepository accountSecurityRepository, LockFactory lockFactory, CurrentService currentService, ThreadPoolTaskExecutor taskExecutor) {
        this.managerRepository = managerRepository;
        this.departmentRepository = departmentRepository;
        this.systemConfigRepository = systemConfigRepository;
        this.accountSecurityRepository = accountSecurityRepository;
        this.lockFactory = lockFactory;
        this.currentService = currentService;
        this.taskExecutor = taskExecutor;
    }

    @Scheduled(cron = "${ldap.sync.schedule:0 0 0 * * ?}")
    public void sync() {
        if (currentService.isLeader()) {
            taskExecutor.execute(this::execute);
        }
    }

    private void execute() {
        String path = "ldapLock";
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemConfig urlConfig = systemConfigRepository.queryByCode("LdapUrl");
            if (null == urlConfig || !StringUtils.hasText(urlConfig.getValue())) {
                return;
            }
            SystemConfig userConfig = systemConfigRepository.queryByCode("LdapUsername");
            if (null == userConfig || !StringUtils.hasText(userConfig.getValue())) {
                return;
            }
            SystemConfig pswdConfig = systemConfigRepository.queryByCode("LdapPassword");
            if (null == pswdConfig || !StringUtils.hasText(pswdConfig.getValue())) {
                return;
            }
            SystemConfig baseConfig = systemConfigRepository.queryByCode("LdapBaseDn");
            if (null == baseConfig || !StringUtils.hasText(baseConfig.getValue())) {
                return;
            }
            LdapConfig config = new LdapConfig(urlConfig.getValue(), userConfig.getValue(), new DESEncrptor().decrypt(pswdConfig.getValue(),"BocloudCMP"),
                    baseConfig.getValue(), "");
//            new LdapTransportor(managerRepository, departmentRepository, accountSecurityRepository, config).sync();
        } catch (Exception e) {
            logger.error("ldap sync error.");
        }
    }

    @Override
    public void afterPropertiesSet() {
//        if (currentService.isLeader()) {
//            taskExecutor.execute(this::execute);
//        }
    }

}
