package com.bocloud.sms.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.sms.model.CloudServiceBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;



/**
 * 服务
 *
 * @author hya
 * @since 2022-03-02
 */
@Repository
public class CloudServiceBeanRepository extends BasicGenericDao<CloudServiceBean, Long> {

    @Autowired
    public CloudServiceBeanRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 获取收藏服务
     *
     * @param userId
     * @return
     */
    public List<CloudServiceBean> getFavoriteServices(Long userId,String category) {
        String sql = "select DISTINCT cs.*,sc.name catalog_name,i.icon icon,f.priority priority,f.id favorite_id from cloud_service cs left join service_catalog sc on cs.catalog_id=sc.id left join icon i on cs.icon_id = i.id left join favorite f on cs.id = f.service_id where f.category = :category and f.target = :userId and cs.code != 'vmware.storage.disk' and cs.status= 'PUBLISHED'";
        Map<String, Object> paramsMap = MapTools.simpleMap("userId", userId);
        paramsMap.put("category",category);
        return super.list(CloudServiceBean.class,sql,paramsMap);
    }

    /**
     * 获取服务菜单
     *
     * @return
     */
    public List<CloudServiceBean> getServiceMenu() {
        String sql = "select DISTINCT cs.*,sc.name catalog_name,i.icon icon from cloud_service cs left join service_catalog sc on cs.catalog_id=sc.id left join icon i on cs.icon_id = i.id where  cs.is_deleted = 0 and cs.status='PUBLISHED' and cs.code !='vmware.storage.disk' ";
        return super.list(CloudServiceBean.class,sql);
    }

}
