package com.bocloud.sms.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.sms.entity.RolePermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * 角色权限关系
 *
 * @author hya
 * @since 2022-03-02
 */
@Repository
public class RolePermissionRepository extends BasicGenericDao<RolePermission, Long> {
    private static final Logger logger = LoggerFactory.getLogger(RolePermissionRepository.class);

    @Autowired
    public RolePermissionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据角色ID查询角色权限
     *
     * @param roleId
     * @return
     * @throws BoCloudDBException
     */
    public List<RolePermission> listPermission(Long roleId) throws BoCloudDBException {
        String sql = "select * from role_permission where role_id = :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return super.list(RolePermission.class,sql,params);
    }

    /**
     * 根据角色ID删除角色权限
     *
     * @param roleId
     * @return
     * @throws BoCloudDBException
     */
    public boolean unBindPermission(Long roleId) throws BoCloudDBException {
        String sql = "delete from role_permission where role_id = :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return super.execute(sql,params) > 0;
    }

    /**
     * 根据权限ID批量删除角色权限
     *
     * @param permissionIds
     * @return
     * @throws BoCloudDBException
     */

    public boolean unBindRole(List<Long> permissionIds) throws BoCloudDBException {
        String sql = "delete from role_permission where permission_id in (:permissionIds)";
        Map<String, Object> params = MapTools.simpleMap("permissionIds", permissionIds);
        return super.execute(sql, params) > 0;
    }


    /**
     * 批量插入角色权限
     *
     * @param rolePermissions
     * @throws BoCloudDBException
     */
    public boolean bindPermissions(List<RolePermission> rolePermissions) {
        try {
            StringBuilder sql = new StringBuilder("insert into role_permission(role_id, permission_id) values ");
            for (int i = 0; i < rolePermissions.size(); i++) {
                sql.append("(").append(rolePermissions.get(i).getRoleId())
                        .append(", ")
                        .append(rolePermissions.get(i).getPermissionId())
                        .append(")");
                if (i < rolePermissions.size() -1) {
                    sql.append(", ");
                }
            }
            super.execute(sql.toString());
            return true;
        } catch (Exception e) {
            logger.error(" 批量插入角色权限异常",e);
            return false;
        }
    }

}
