package com.bocloud.sms.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.sms.entity.SystemTask;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 平台任务数据访问
 *
 * @author tyl
 * @since 2021/4/1 17:19
 */
@Repository("systemTaskRepository")
public class SystemTaskRepository extends JdbcGenericDao<SystemTask, Long> {

    public SystemTaskRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据id删除平台任务
     *
     * @param id
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean delete(Long id, Long menderId) throws BoCloudDBException {
        String sql = "update system_task set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from system_task where is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        List<SystemTask> systemTasks = this.list(SystemTask.class, sql, param);
        return systemTasks.stream().map(systemTask -> new SimpleBean(systemTask.getId(), systemTask.getName())).collect(Collectors.toList());
    }


    /**
     * 分页查询
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     * @throws BoCloudDBException
     */
    public List<SystemTask> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from system_task where is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.list(SystemTask.class, sql, param);
    }


    /**
     * 获取平台数量
     *
     * @param params
     * @return
     * @throws BoCloudDBException
     */
    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "select count(st.id) from system_task st where st.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "st");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 获取平台任务详情
     *
     * @param id
     * @return
     * @throws BoCloudDBException
     */
    public SystemTask detail(Long id) throws BoCloudDBException {
        String sql = "select * from system_task where is_deleted = 0 and id = :id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<SystemTask> list = this.list(SystemTask.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据名称查询平台任务
     *
     * @param name
     * @return
     * @throws BoCloudDBException
     */
    public SystemTask queryByName(String name) throws BoCloudDBException {
        String sql = "select * from system_task where is_deleted = 0 and name = :name ";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<SystemTask> list = this.list(SystemTask.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }
}
